import React, { PureComponent } from 'react';
import ReactNative, {
    View,
	Text,
	Platform,
    requireNativeComponent,
    UIManager,
	ListView,
    SectionList,
} from 'react-native';

import PropTypes from 'prop-types';

import Refreshing from './Refreshing';
import LoadingMore from './LoadingMore';

import RecyclerView from '../recyclerview/RecyclerView';

/**
 *
 */
const PullRefreshHeaderContainerView = requireNativeComponent('AndroidPullRefreshHeaderContainerView');
/**
 *
 */
const PullRefreshFooterContainerView = requireNativeComponent('AndroidPullRefreshFooterContainerView');
/**
 *
 */
const PullRefreshFooterEndContainerView = requireNativeComponent('AndroidPullRefreshFooterEndContainerView');

/**
 *
 */
const NativePullToRefreshContainer = requireNativeComponent('AndroidPullToRefreshContainer', PullToRefreshContainer);

/**
 *
 */
const CONSTANTS_REFRESH_MODE = UIManager.AndroidPullToRefreshContainer.Constants.REFRESH_MODE;

/**
 *
 */
class PullToRefreshContainer extends PureComponent{

    /**
     *
     * @type {{useRefreshViewDefault: React$PropType$Primitive.<boolean>, refreshMode: *, headerRefreshView, footerRefreshView, onPullEvent, onPullDownToRefresh, onPullUpToRefresh}}
     */
    static propTypes = {
        refreshMode:PropTypes.oneOf([CONSTANTS_REFRESH_MODE.DISABLED, CONSTANTS_REFRESH_MODE.PULL_FROM_START, CONSTANTS_REFRESH_MODE.PULL_FROM_END, CONSTANTS_REFRESH_MODE.BOTH]),   // 下拉/上拉刷新模式 默认: DISABLED
        useRefreshViewDefault:PropTypes.bool, //是否原生使用默认的刷新view  默认：true
        headerRefreshView:PropTypes.element,   //自定义下拉刷新view
        headerRefreshViewHeight:PropTypes.number,   //下拉刷新View的高度
        footerRefreshView:PropTypes.element,   //自定义上拉刷新view
        footerEndView:PropTypes.element,   //自定义加载完成view
        onPullEvent:PropTypes.func,   //下拉状态通知
        onPullDownToRefresh:PropTypes.func,   //下拉刷新
        onPullUpToRefresh:PropTypes.func,   //上拉加载
        autoLoadMore:PropTypes.bool, //自动加载更多
    }

    /**
     *
     * @type {{useRefreshViewDefault: boolean, refreshMode: *}}
     */
    static defaultProps = {
        refreshMode:CONSTANTS_REFRESH_MODE.PULL_FROM_START,
        useRefreshViewDefault:true,
        headerRefreshViewHeight:0,  //0：表示原生默认高度
        autoLoadMore:true,
    }

    /**
	 *
     */
    static REFRESH_MODE = CONSTANTS_REFRESH_MODE;

    /**
     * 是否下拉刷新
     * @type {null}
     * @private
     */
    _isPullDownToRefresh = null;

	constructor(props){
		super(props);
	}

    /**
     * 刷新完成
     */
    onRefreshComplete = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidPullToRefreshContainer.Commands.onRefreshComplete, null);
        }
    }

    /**
     * 自动下拉刷新
     */
    setTopRefreshing = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidPullToRefreshContainer.Commands.setTopRefreshing, null);
        }
    }

    /**
     * 底部加载结束
     */
    onLoadMoreComplete = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidPullToRefreshContainer.Commands.onLoadMoreComplete, null);
        }
    }

    /**
     * 没有更多
     */
    onLoadMoreCompleteAndNoData = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidPullToRefreshContainer.Commands.onLoadMoreCompleteAndNoData, null);
        }
    }

    /**
     *
     * @private
     */
    _getListViewHandle = ()=>{
        if(this.refs.nativePullToRefreshContainer){
            return ReactNative.findNodeHandle(this.refs.nativePullToRefreshContainer);
        }
    }

    /**
     * 下拉/上拉 状态通知
     * @param e
     * @private
     */
    _onPullEvent = (e)=>{
        this._isPullDownToRefresh = false;

        this.props.onPullEvent && this.props.onPullEvent(e.nativeEvent);
    }

	render(){

        let { headerRefreshView, footerRefreshView, footerEndView, refreshMode, autoLoadMore, useRefreshViewDefault, children, ...otherProps } = this.props;

        let pullRefreshHeaderContainerView;
        //下拉刷新view
        if(!useRefreshViewDefault && (refreshMode === CONSTANTS_REFRESH_MODE.BOTH || refreshMode === CONSTANTS_REFRESH_MODE.PULL_FROM_START) && headerRefreshView){
            pullRefreshHeaderContainerView = (
                <PullRefreshHeaderContainerView>
                    {headerRefreshView}
                </PullRefreshHeaderContainerView>
            );
        }

        //上拉刷新view 或 自动刷新view
        let pullRefreshFooterContainerView = null;
        if(!useRefreshViewDefault && (refreshMode === CONSTANTS_REFRESH_MODE.BOTH || refreshMode === CONSTANTS_REFRESH_MODE.PULL_FROM_END) && footerRefreshView){
            let pullRefreshFooterEndContainerView = null;
            if(footerEndView){
                pullRefreshFooterEndContainerView = (
                    <PullRefreshFooterEndContainerView>
                        {footerEndView}
                    </PullRefreshFooterEndContainerView>
                );
            }

            pullRefreshFooterContainerView = (
                <PullRefreshFooterContainerView>
                    {footerRefreshView}
                    {pullRefreshFooterEndContainerView}
                </PullRefreshFooterContainerView>
            );
        }

        if(!useRefreshViewDefault && autoLoadMore && footerRefreshView){
            pullRefreshFooterContainerView = (
                <PullRefreshFooterContainerView>
                    {footerRefreshView}
                </PullRefreshFooterContainerView>
            );
        }

		return (
            <NativePullToRefreshContainer
                ref={'nativePullToRefreshContainer'}
                {...otherProps}
                refreshMode={refreshMode}
                useRefreshViewDefault={useRefreshViewDefault}
                autoLoadMore={autoLoadMore}
                onPullEvent={this._onPullEvent}
            >
                { pullRefreshHeaderContainerView }

                { children }

                { pullRefreshFooterContainerView }

            </NativePullToRefreshContainer>
        );
	}

}








/**
 * 下拉头部最大高度
 * @type {number}
 */
const MAX_REFRESH_HEIGHT = 50;

/**
 *
 */
export default class RefreshListView extends PureComponent {

    /**
	 *
     * @type {{useRefreshViewDefault: React$PropType$Primitive.<boolean>, enabledPullDown: React$PropType$Primitive.<boolean>, enabledPullUp: React$PropType$Primitive.<boolean>, onRefresh, onLoadMore}}
     */
    static propTypes = {
        enabledPullDown:PropTypes.bool,		//是否开启下拉刷新
        enabledPullUp:PropTypes.bool,		//是否开启上拉加载
        onRefresh:PropTypes.func,   //下拉刷新
        onLoadMore:PropTypes.func,   //上拉加载
    }

    /**
	 *
     * @type {{enabledPullDown: boolean, enabledPullUp: boolean}}
     */
    static defaultProps = {
        enabledPullDown: false,
        enabledPullUp: false
    }

    /**
     *
     * @type {null}
     * @private
     */
    _ds = null;
    /**
     *
     * @type {Array}
     * @private
     */
    _dataList = [];
    /**
     * 是否正在下拉刷新
     * @type {boolean}
     */
    isRefreshing = false;
    /**
     * 是否正在加载更多
     * @type {boolean}
     */
    isLoadMore = false;

    constructor(props) {
        super(props);

        const { dataSource, dataList } = this.props;

        if(dataSource){
            this.state = {
                dataSource:dataSource,
            }
        }else{
            this._dataList.addAll(dataList);
            this._ds = new ListView.DataSource({rowHasChanged: (r1, r2) => r1 !== r2});
            this.state={
                dataSource:this._ds.cloneWithRows(this._dataList),
            }
        }
    }

    /**
     *
     * @param dataSource
     */
    setDataSource = (dataSource : ListView.DataSource)=>{
        this.setState({ dataSource });
    }

    /**
     *
     * @param item
     * @param index
     */
    addItemData = (item : any, index : number)=>{
        if(this._ds){
            this._dataList.add(item, index);
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     *
     * @param item
     */
    addAllItemData = (itemArray : Array) => {
        if(this._ds){
            this._dataList.addAll(itemArray);
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     *
     * @param position
     */
    removeItemDataAtPosition = (position : number)=>{
        if(this._ds){
            this._dataList.remove(position);
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     *
     * @param position
     */
    removeItemData = (item : any)=>{
        if(this._ds){
            this._dataList.remove(item);
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     * 清空数据
     */
    clear = ()=>{
        if(this._ds){
            this._dataList = [];
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     * 下拉/上拉 状态通知
     * @param e
     * @private
     */
    _onPullEvent = ({ currMode, pullState })=>{
        this.isRefreshing = false;
        this.isLoadMore = false;

        let refreshTxt = '';

        if(currMode === 'AUTO_LOAD_MORE'){

            if(pullState === 'NORMAL'){
                refreshTxt = '自动加载更多';
                this._changeLoadMoreState(null, false);
            }else if(pullState === 'LOADMORE_LOADING'){
                refreshTxt = '加载中...';
                this.isLoadMore = true;
                this._changeLoadMoreState(null, false);
            }else if(pullState === 'LOADMORE_COMPLETE'){
                refreshTxt = '加载完成';
                this._changeLoadMoreState(null, false);
            }else if(pullState === 'LOADMORE_NOMORE'){
                refreshTxt = '没有更多了';
                this._changeLoadMoreState(null, true);
            }else if(pullState === 'LOADMORE_FOOTER_GONE'){
                refreshTxt = '加载更多隐藏';
            }
        }else if(currMode === 'PULL_FROM_START'){ //下拉
            this.refs.headerRefreshView && this.refs.headerRefreshView.stopAnim();

            if(pullState === 'PULL_TO_REFRESH'){
                refreshTxt = '下拉即可刷新';
            }else if(pullState === 'RELEASE_TO_REFRESH'){
                refreshTxt = '释放即可刷新';
            }else if(pullState === 'RESET'){
                refreshTxt = '下拉即可刷新';
            }else if(pullState === 'REFRESHING'){
                refreshTxt = '刷新中...';
                this.isRefreshing = true;
                this.refs.headerRefreshView && this.refs.headerRefreshView.startAnim();
            }
        }else{  //上拉
            if(pullState === 'PULL_TO_REFRESH'){
                refreshTxt = '上拉即可刷新';
            }else if(pullState === 'RELEASE_TO_REFRESH'){
                refreshTxt = '释放即可刷新';
            }else if(pullState === 'RESET'){
                refreshTxt = '上拉即可刷新';
            }else if(pullState === 'REFRESHING'){
                refreshTxt = '刷新中...';
                this.isLoadMore = true;
            }
        }
        console.log('_onPullEvent refreshTxt============> ', refreshTxt, '  pullState: ', pullState);
    }

    /**
	 * 下拉刷新开始
     * @private
     */
    _onPullDownToRefresh = ()=>{
    	console.log('下拉刷新开始===========>');
        this.props.onRefresh && this.props.onRefresh();
	}

    /**
	 * 上拉刷新开始
     * @private
     */
    _onPullUpToRefresh = ()=>{
        console.log('上拉刷新开始===========>');
        this.props.onLoadMore && this.props.onLoadMore();
	}

    /**
     * 自动下拉刷新
     */
    beginRefreshing = () => {
        if (!this.isRefreshing && !this.isLoadMore) {
            this.refs.pullToRefreshContainer && this.refs.pullToRefreshContainer.setTopRefreshing();
            this.isRefreshing = true;
        }
    }

    /**
     * 结束下拉刷新
     */
    endRefreshing = () => {
        if (this.isRefreshing) {
            this.refs.pullToRefreshContainer && this.refs.pullToRefreshContainer.onRefreshComplete();
            this.isRefreshing = false;
        }
    }

    /**
     * 上拉加载结束
     * @param hasLoadAll    是否已经加载所有数据  true：底部显示没有更多了  false: 加载中...
     */
    endLoadMore = (hasLoadAll : boolean = false) => {
        if(this.refs.pullToRefreshContainer && !this.refs.pullToRefreshContainer._isPullDownToRefresh){
            if(hasLoadAll){	//没有更多
                this.refs.pullToRefreshContainer.onLoadMoreCompleteAndNoData();
            }else{	//加载更多结束
                this.refs.pullToRefreshContainer.onLoadMoreComplete();
            }
        }
    }

    /**
     *
     * @type {e}
     * @private
     */
    _pullToRefreshContainerLayout = (e)=>{
        const { nativeEvent } = e;
        this.refs.recyclerView && this.refs.recyclerView.setNativeProps({
            style:{
                flex:0,
                height:nativeEvent.layout.height
            }
        })
        this.refs.recyclerContainerView && this.refs.recyclerContainerView.setNativeProps({
            style:{
                flex:0,
                height:nativeEvent.layout.height
            }
        })
    }

    /**
     * 头部刷新 View
     * @param txt
     * @returns {XML}
     * @private
     */
    _renderHeaderRefreshView = (txt : String, play = false)=>{
        return (
            <Refreshing
                key={'headerRefreshView'}
                ref={'headerRefreshView'}
                style={{height: 50}}
                play={play}
                isShow={true}
                text={txt}
            />
        );
    }

    /**
     * 加载更多 View
     * @param txt
     * @param hasLoadAll    是否已经加载全部数据   true：显示没有更多了 false：显示正在加载中...
     * @returns {XML}
     * @private
     */
    _renderLoadMoreView = (txt : String = '盟管家正在加载', hasLoadAll : Boolean = false)=>{

        const { loadAllLineStyle } = this.props;

        return (
            <LoadingMore
                key={'loadMoreView'}
                ref={'loadMoreView'}
                style={{height: 50}}
                isShow={true}
                loadAllLineStyle={loadAllLineStyle}
                hasLoadAll={hasLoadAll}
                txt={txt}
            />
        );
    }

    /**
     * 切换加载更多状态
     * @param String
     * @param hasLoadAll
     * @private
     */
    _changeLoadMoreState = (txt : String = '盟管家正在加载', hasLoadAll : Boolean = false)=>{
        this.refs.loadMoreView && this.refs.loadMoreView.changeState(txt, hasLoadAll);
    }

	render() {

		const { enabledPullUp, enabledPullDown, onRefresh, onLoadMore, style, contentContainerStyle, scrollEventThrottle = 200, ...otherProps } = this.props;

		const { dataSource } = this.state;

		let autoLoadMore = false;
		let refreshMode = PullToRefreshContainer.REFRESH_MODE.DISABLED;
		if(!!enabledPullDown && !!enabledPullUp){	//支持下拉、上拉
			refreshMode = PullToRefreshContainer.REFRESH_MODE.PULL_FROM_START;
            autoLoadMore = true;
		}else if(!!enabledPullDown){	//下拉
			refreshMode = PullToRefreshContainer.REFRESH_MODE.PULL_FROM_START;
            autoLoadMore = false;
		}else if(!!enabledPullUp){	//上拉
			refreshMode = PullToRefreshContainer.REFRESH_MODE.DISABLED;
            autoLoadMore = true;
		}

		//
		const refreshControl = (
            <PullToRefreshContainer
                onLayout={this._pullToRefreshContainerLayout}
                ref={'pullToRefreshContainer'}
                style={{flex:1}}
                refreshMode={refreshMode}
                onPullEvent={this._onPullEvent}
                onPullDownToRefresh={this._onPullDownToRefresh}
                onPullUpToRefresh={this._onPullUpToRefresh}
                useRefreshViewDefault={false}
                headerRefreshView={this._renderHeaderRefreshView()}
                headerRefreshViewHeight={MAX_REFRESH_HEIGHT}
                footerRefreshView={this._renderLoadMoreView()}
                footerEndView={this._renderLoadMoreView(null, true)}
                footerRefreshViewHeight={MAX_REFRESH_HEIGHT}
                autoLoadMore={autoLoadMore}
            />
        );

        const dataList = [];
        if(dataSource){
            for(let sectionIndex = 0; sectionIndex < dataSource.sectionIdentities.length; sectionIndex++){
                let sectionID = dataSource.sectionIdentities[sectionIndex];
                let sectionHeaderData = dataSource.getSectionHeaderData(sectionIndex);
                let sectionHeaderItem = {
                    sectionID:sectionID,
                    data:sectionHeaderData,
                };
                // dataList.add(sectionHeaderItem);

                let sectionRowIdentities = dataSource.rowIdentities[sectionIndex];
                for(let sectionRowIndex = 0; sectionRowIndex < sectionRowIdentities.length; sectionRowIndex++){
                    let rowID = sectionRowIdentities[sectionRowIndex];
                    let rowData = dataSource.getRowData(sectionIndex, sectionRowIndex);
                    let rowItem = {
                        sectionID:sectionID,
                        rowID:rowID,
                        data:rowData,
                    };
                    dataList.add(rowItem);
                }
            }
        }

        const recyclerViewStyle = [style, contentContainerStyle];

        const recyclerView = (
            <View
                ref={'recyclerContainerView'}
                pointerEvents={'box-none'}
                style={[styles.container, recyclerViewStyle]}
                opacity={1}
            >
                <RecyclerView
                    ref={'recyclerView'}
                    {...otherProps}
                    scrollEventThrottle={scrollEventThrottle}
                    style={styles.list}
                    itemCount={dataList.length}
                    dataList={dataList}
                />
            </View>
        );

        if(!enabledPullDown && !enabledPullUp){
            return recyclerView;
        }

        return React.cloneElement(refreshControl, null, recyclerView);
	}
}

const styles = {
    container:{
        flex:1,
    },
    list:{
        flex:1,
    }
}